"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.hasPermissions = void 0;
const bf_types_1 = require("bf-types");
const system_1 = __importStar(require("../system"));
const common_1 = require("../common");
function getUser() {
    return __awaiter(this, void 0, void 0, function* () {
        return system_1.default.nexus.getUser();
    });
}
function getUserDoc() {
    return __awaiter(this, void 0, void 0, function* () {
        const api = system_1.default.getLibModule(system_1.LibModule.API);
        const userId = system_1.default.nexus.getUser().sub;
        const personId = system_1.default.nexus.getUser().person_id;
        const user = yield api.get(`core/user/entity/${userId}`);
        if (personId && !user) {
            const person = yield api.get(`crm/people/entity/${personId}`);
            if (person) {
                return person;
            }
        }
        if (!user) {
            throw new Error('Failed to retrieve the document for the authenticated user.');
        }
        return user;
    });
}
function getUserDocs() {
    return __awaiter(this, void 0, void 0, function* () {
        const api = system_1.default.getLibModule(system_1.LibModule.API);
        const users = yield api.get('users');
        if (!users || !Array.isArray(users)) {
            return [];
        }
        return users;
    });
}
function getOrganization() {
    return __awaiter(this, void 0, void 0, function* () {
        return common_1.moduleLink(bf_types_1.CORE_MODULES.ORGANIZATION, system_1.default.nexus.getUser().organization[0]);
    });
}
function getOrganizationDoc() {
    return __awaiter(this, void 0, void 0, function* () {
        const api = system_1.default.getLibModule(system_1.LibModule.API);
        const organizationId = system_1.default.nexus.getUser().organization[0];
        const organization = yield api.get(`core/organization/entity/${organizationId}`);
        if (!organization) {
            throw new Error("failed to retrieve the document for the authenticated user's organization");
        }
        return organization;
    });
}
function hasPermissions(participants, securityLevel, ...validModules) {
    let state = false;
    const scopes = [];
    const validModuleIds = validModules.map((m) => m.module_id);
    for (const participant of participants) {
        if (validModuleIds.includes(participant.module_id)) {
            scopes.push(...participant.scopes);
        }
    }
    for (const scope of scopes) {
        // if disallows, return immediately
        if (bf_types_1.ScopeDefinition[scope].disallows.indexOf(securityLevel) !== -1) {
            return false;
        }
        // if allowed set state to true, but continue looking, in case a scope disallows it, Disallows trumps allows
        if (bf_types_1.ScopeDefinition[scope].allows.indexOf(securityLevel) !== -1) {
            state = true;
        }
    }
    return state;
}
exports.hasPermissions = hasPermissions;
function logOut() {
    system_1.default.nexus.disconnect();
}
const auth = {
    getUser,
    getUserDoc,
    getUserDocs,
    getOrganizationDoc,
    getOrganization,
    hasPermissions,
    logOut,
};
exports.default = system_1.default.sealModule(Object.freeze(auth));
