"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.formTemplateToChanges = exports.resolveBracketSyntax = exports.resolveBracketStatergy = exports.castValueFromContext = exports.convertToTemplateBidningFromDb = exports.convertFormTemplateBindingFromDb = exports.resolveOutputBindingValue = exports.resolveUnmanipulatedOutputValue = exports.deepResolve = void 0;
const handlebars_1 = __importDefault(require("@timwoods/handlebars"));
const deepmerge_1 = __importDefault(require("deepmerge"));
const lodash_isequal_1 = __importDefault(require("lodash.isequal"));
const uuid_1 = require("uuid");
const FormOutputManipulator_1 = require("./FormOutputManipulator");
const deepResolve = (fullkey, object) => {
    const keys = fullkey.split('.');
    let current = Object.assign({}, object);
    for (const key of keys) {
        if (typeof current === 'object' && current) {
            current = current[key];
        }
    }
    return current;
};
exports.deepResolve = deepResolve;
function combineMerge(target, source, options) {
    const destination = target.slice();
    source.forEach((item, index) => {
        if (typeof destination[index] === 'undefined') {
            destination[index] = options.cloneUnlessOtherwiseSpecified(item, options);
        }
        else if (options.isMergeableObject(item)) {
            destination[index] = deepmerge_1.default(target[index], item, options);
        }
        else if (target.indexOf(item) === -1) {
            destination.push(item);
        }
    });
    return destination;
}
const resolveUnmanipulatedOutputValue = (binding, fullContext) => {
    switch (binding.type) {
        case 'CONTEXT':
            const value = exports.deepResolve(`${binding.value.data_source}.${binding.value.data_key}`, fullContext);
            if (value || !binding.default) {
                return { value, source: 'value', manipulators: binding.value.manipulators };
            }
            const defaultValue = exports.deepResolve(`${binding.default.data_source}.${binding.default.data_key}`, fullContext);
            return { value: defaultValue, source: 'default', manipulators: binding.default.manipulators };
        case 'CONTEXT_TEMPLATE': {
            const value = handlebars_1.default.compile(binding.value)(fullContext);
            return { value, source: 'value' };
        }
        case 'VALUE':
            return { value: binding.value, source: 'value' };
    }
};
exports.resolveUnmanipulatedOutputValue = resolveUnmanipulatedOutputValue;
const resolveOutputBindingValue = (binding, fullContext) => {
    return FormOutputManipulator_1.getManipulatedValue(binding, fullContext);
};
exports.resolveOutputBindingValue = resolveOutputBindingValue;
const convertFormTemplateBindingFromDb = (formTemplate) => {
    const clean = {};
    for (const key of Object.keys(formTemplate)) {
        clean[key.replace(/,/g, '.')] = formTemplate[key];
    }
    return clean;
};
exports.convertFormTemplateBindingFromDb = convertFormTemplateBindingFromDb;
const convertToTemplateBidningFromDb = (formTemplate) => {
    const clean = {};
    for (const key of Object.keys(formTemplate)) {
        clean[key.replace(/\./g, ',')] = formTemplate[key];
    }
    return clean;
};
exports.convertToTemplateBidningFromDb = convertToTemplateBidningFromDb;
// eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
function castValueFromContext(contextKeyValue, matchValue) {
    switch (typeof contextKeyValue) {
        case 'number':
            return parseInt(matchValue, 10);
        case 'boolean': {
            if (matchValue === 'true') {
                return true;
            }
            else if (matchValue === 'false') {
                return false;
            }
        }
        default:
            return matchValue;
    }
}
exports.castValueFromContext = castValueFromContext;
function resolveBracketStatergy(postKey) {
    const regex = /^(.*)?\[(.*)\]/gi;
    const [stg, key] = [postKey.replace(regex, '$1'), postKey.replace(regex, '$2')];
    const validStratergySymbols = ['_', '+', '~'];
    const stratergyValue = {
        stratergy: key.length === 0 ? 'REPLACE' : 'UPDATE',
        postKey: key,
    };
    if (!postKey.startsWith('[') && validStratergySymbols.includes(stg) && key.length !== 0) {
        const stratergySymbol = postKey.charAt(0);
        stratergyValue.postKey = postKey.substring(1).replace(/[\[\]]/g, '');
        switch (stratergySymbol) {
            case '+':
                stratergyValue.stratergy = 'UPSERT';
                break;
            case '_':
                stratergyValue.stratergy = 'UPDATE';
                break;
            case '~':
                stratergyValue.stratergy = 'REPLACE';
                break;
        }
    }
    return stratergyValue;
}
exports.resolveBracketStatergy = resolveBracketStatergy;
function resolveBracketSyntax(target, keys, bindingValues, fullContext) {
    const preKey = keys.slice(0, -1).join('.');
    const { postKey, stratergy } = resolveBracketStatergy(keys[keys.length - 1]);
    const matchers = postKey.split(';').map((subMatcher) => subMatcher.split(':'));
    let contextValues = exports.deepResolve(`${target}.${preKey}`, fullContext) || [];
    bindingValues = Array.isArray(bindingValues) ? bindingValues : [bindingValues];
    if (!Array.isArray(contextValues)) {
        contextValues = [];
    }
    const resolveValue = (changeActionSymbol, value, mergeValue = {}) => {
        const key = (['REPLACE', 'UPSERT'].includes(stratergy) && !postKey ? [['id']] : matchers)
            .map((matcher) => {
            if (matcher.length === 1) {
                const resolvedContextValue = exports.deepResolve(matcher[0], value) || uuid_1.v4();
                return [
                    ...matcher,
                    typeof resolvedContextValue === 'object' && !Array.isArray(resolvedContextValue)
                        ? JSON.stringify(resolvedContextValue)
                        : resolvedContextValue,
                ].join(':');
            }
            return matcher.join(':');
        })
            .join(';');
        return {
            [`${preKey}.${changeActionSymbol}[${key}]`]: deepmerge_1.default(mergeValue, value, {
                arrayMerge: combineMerge,
            }),
        };
    };
    const isFoundHistoryHistory = [];
    const _changeActions = bindingValues.flatMap((bindingValue) => {
        const changeActions = [];
        const isFoundHistory = [];
        for (const contextValue of contextValues) {
            const isFound = matchers.reduce((pv, cv) => {
                if (cv.length === 1) {
                    const key = cv[0];
                    const resolvedBindingValue = exports.deepResolve(key, bindingValue);
                    const resolvedContextValue = exports.deepResolve(key, contextValue);
                    return pv && lodash_isequal_1.default(resolvedBindingValue, resolvedContextValue);
                }
                else if (cv.length === 2) {
                    const contextKeyValue = exports.deepResolve(cv[0], contextValue);
                    return pv && lodash_isequal_1.default(contextKeyValue, castValueFromContext(contextKeyValue, cv[1]));
                }
                return pv && false;
            }, !postKey && stratergy === 'REPLACE' ? false : true);
            isFoundHistory.push(isFound);
            if (stratergy === 'UPDATE' && isFound) {
                changeActions.push(resolveValue('_', bindingValue, contextValue));
            }
            else if (stratergy === 'UPSERT') {
                // Update by merging
                if (isFound) {
                    changeActions.push(resolveValue('_', bindingValue, contextValue));
                }
            }
            else if (stratergy === 'REPLACE') {
                // Update by replacing and all timwood module properties have ids
                if (isFound) {
                    changeActions.push(resolveValue('_', bindingValue, { id: contextValue.id }));
                }
            }
        }
        const isSomeFound = isFoundHistory.some((isFound) => isFound);
        isFoundHistoryHistory.push(isFoundHistory);
        if (['REPLACE', 'UPSERT'].includes(stratergy) && !isSomeFound) {
            // Insert if atleast a single value didn't match
            changeActions.push(resolveValue('+', bindingValue));
        }
        return changeActions;
    });
    // Remove all old items that didn't match any binding value
    if (stratergy === 'REPLACE') {
        for (let i = 0; i < contextValues.length; i++) {
            const isSomeFoundHorizontally = isFoundHistoryHistory.some((isFoundHistory) => isFoundHistory[i]);
            if (!isSomeFoundHorizontally) {
                _changeActions.push(resolveValue('-', contextValues[i]));
            }
        }
    }
    return _changeActions;
}
exports.resolveBracketSyntax = resolveBracketSyntax;
const formTemplateToChanges = (form_data, context, formTemplate, create) => {
    var _a, _b;
    const fullContext = Object.assign(Object.assign({}, context), { form_data });
    const changes = [];
    const form_output_bindings = exports.convertFormTemplateBindingFromDb(formTemplate.form_output_bindings);
    const entity = {
        module: {
            module_name: ((_a = context.entity) === null || _a === void 0 ? void 0 : _a.module_name) || formTemplate.linked_module_name,
            module_id: ((_b = context.entity) === null || _b === void 0 ? void 0 : _b.id) || 'create',
        },
        changes: {},
        create,
    };
    const creator = {
        module: {
            module_name: context.creator.module_name,
            module_id: context.creator.id,
        },
        changes: {},
    };
    const actor = {
        module: {
            module_name: context.actor.module_name,
            module_id: context.actor.id,
        },
        changes: {},
    };
    function updateChanges(target, subkey, value) {
        switch (target) {
            case 'creator':
                creator.changes[subkey] = value;
                break;
            case 'entity':
                entity.changes[subkey] = value;
                break;
            case 'actor':
                actor.changes[subkey] = value;
                break;
        }
    }
    for (let option of Object.keys(form_output_bindings)) {
        option = option.trim();
        const binding = form_output_bindings[option];
        const value = exports.resolveOutputBindingValue(binding, fullContext);
        const keys = option.split(/\.(?![^[]*])/gi);
        const target = keys.shift();
        const subkey = keys.join('.');
        if (/.*\[.*\]$/.test(option) || Array.isArray(value)) {
            const bracketSyntaxChanges = resolveBracketSyntax(target, [...keys, ...(/.*\[.*\]$/.test(option) ? [] : ['~[]'])], value, fullContext);
            for (const change of bracketSyntaxChanges) {
                for (const [key, value] of Object.entries(change)) {
                    updateChanges(target, key, value);
                }
            }
        }
        else {
            updateChanges(target, subkey, value);
        }
    }
    changes.push(entity);
    if (Object.keys(actor.changes).length !== 0) {
        changes.push(actor);
    }
    if (Object.keys(creator.changes).length !== 0) {
        changes.push(creator);
    }
    return changes;
};
exports.formTemplateToChanges = formTemplateToChanges;
