"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.hrsToMs = exports.getHoursFromMs = exports.formatMs = exports.debounce = exports.randomDelay = exports.haveSameMinute = exports.minutesToMs = exports.secondsToMs = exports.delay = exports.withDelay = exports.sleep = void 0;
const number_utils_1 = require("./number.utils");
/**
 * Return a promise that will resolve after the given number of milliseconds
 *
 * @param time The time in milliseconds to wait before resolving the promise
 */
function sleep(time) {
    return new Promise((done) => {
        setTimeout(done, time);
    });
}
exports.sleep = sleep;
function withDelay(ms) {
    return (callback) => delay(ms, callback);
}
exports.withDelay = withDelay;
function delay(ms, callback) {
    return () => new Promise((resolve) => setTimeout(resolve, ms)).then(callback);
}
exports.delay = delay;
function secondsToMs(seconds) {
    return seconds * 1000;
}
exports.secondsToMs = secondsToMs;
function minutesToMs(minutes) {
    return minutes * 60000;
}
exports.minutesToMs = minutesToMs;
/**
 * Check to see if two UNIX timestamps have the same minute value
 */
function haveSameMinute(left, right) {
    return new Date(left).getMinutes() === new Date(right).getMinutes();
}
exports.haveSameMinute = haveSameMinute;
function randomDelay(min, max) {
    return (callback) => new Promise((resolve) => setTimeout(resolve, number_utils_1.getRandomInt(min, max))).then(callback);
}
exports.randomDelay = randomDelay;
function debounce(cb, wait = 20) {
    let timeout = null;
    const callable = (...args) => {
        if (timeout) {
            clearTimeout(timeout);
        }
        timeout = setTimeout(() => cb(...args), wait);
    };
    return callable;
}
exports.debounce = debounce;
function formatMs(ms) {
    if (ms <= 0) {
        return 'N/a';
    }
    if (ms <= 1000) {
        return '1s';
    }
    const seconds = ms / 1000;
    if (seconds < 60) {
        return String(Math.floor(seconds)) + 's';
    }
    const minutes = seconds / 60;
    if (minutes < 60) {
        return String(Math.floor(minutes)) + 'm';
    }
    const hours = minutes / 60;
    if (hours < 24) {
        return String(Math.floor(hours)) + 'h';
    }
    const days = hours / 24;
    return String(Math.floor(days)) + 'd';
}
exports.formatMs = formatMs;
function getHoursFromMs(ms) {
    return Math.floor(ms / 3600000);
}
exports.getHoursFromMs = getHoursFromMs;
function hrsToMs(hrs) {
    return hrs * 3600000;
}
exports.hrsToMs = hrsToMs;
